/*
 * ***************************************************************
 * COPYRIGHT. HSBC HOLDINGS PLC 2004. ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS ONLY TO BE USED FOR THE PURPOSE FOR WHICH IT
 * HAS BEEN PROVIDED. NO PART OF IT IS TO BE REPRODUCED,
 * DISASSEMBLED, TRANSMITTED, STORED IN A RETRIEVAL SYSTEM NOR
 * TRANSLATED IN ANY HUMAN OR COMPUTER LANGUAGE IN ANY WAY OR
 * FOR ANY OTHER PURPOSES WHATSOEVER WITHOUT THE PRIOR WRITTEN
 * CONSENT OF HSBC HOLDINGS PLC.
 * ***************************************************************
 */
package org.intellij.vcs.mks;

import junit.framework.TestCase;
import mks.integrations.common.*;

/**
 * files not UNDER a sandbox throw exception when validating sandbox (normal, sandbox is not found ?)
 * <p/>
 * un fichier dans une sandbox (le repertoire) mais pas dans le repo
 * file.isStatusControlled() == false
 * un fichier hors d'une sandbox renvoie des exceptions
 *
 * @author Thibaut Fagart
 */
public class TestMks extends TestCase {
	private static final boolean DEBUG = false;

	private static final String SANDBOX_ROOT = "C:/Documents and Settings/A6253567.HBEU/sandboxes/J2EE";
	//	"c:/tfagart/mks/J2EE";
	private static final String DIR_NOT_IN_SANDBOX_MKS = "C:/Documents and Settings";
	private static final String DIR_IN_SANDBOX_MKS = SANDBOX_ROOT + "/HJF-Core";
	private static final String DIR_IN_SANDBOX_MKS_WITH_SLASH = SANDBOX_ROOT + "/HJF-Core/";
	private static final String FILE_NOT_IN_SANDBOX_MKS = "c:/edit2.log";
	private static final String FILE_IN_SANDBOX_MKS =
		SANDBOX_ROOT + "/HJF-Core/src/com/hsbc/hbfr/ccf/at/dao/AbstractDAOFactory.java";
	private static final String FILE_IN_SANDBOX_MODIFIED_MKS = SANDBOX_ROOT + "/HJF-Core/src/META-INF/MANIFEST.MF";
	private static final String FILE_IN_SANDBOX_LOCALLY_DELETED_MKS =
		SANDBOX_ROOT + "/HJF-Core/src/com/hsbc/hbfr/ccf/at/dao/IDataAccesObject.java";
	private static final String FILE_UNDER_NOT_IN_SANDBOX_MKS =
		SANDBOX_ROOT + "/HJF-Core/build/HJF-Core_javadoc.zip";
	private static final String FILE_UNDER_DIR_NOT_IN_SANDBOX_MKS =
		SANDBOX_ROOT + "/HJF-Core/bin/log4j.properties";
//	private static final String FILE_IN_SANDBOX_OLD_REVISION_MKS = "";

	private TriclopsSiSandbox sandbox;
	private static final String DIR_IN_SANDBOX_MKS_NOT_CONTROLLED = SANDBOX_ROOT + "/HJF-Core/bin";
	private static TriclopsSiClient CLIENT = null;

	static {
		System.loadLibrary("mkscmapi");
		CLIENT = new TriclopsSiClient();
		try {
			if (!CLIENT.isIntegrityClientRunning()) {
				CLIENT.initialize("IntelliJ IDEA", 1, 1);
			}
		} catch (TriclopsException e) {
			e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
		}
	}

	protected void setUp() throws Exception {
		super.setUp();
		sandbox = new TriclopsSiSandbox(CLIENT);
	}

	public void testDirNotInSandbox() throws TriclopsException {
		try {
			testFileUnderSandbox(DIR_NOT_IN_SANDBOX_MKS);
			fail();
		} catch (TriclopsException e) {
			System.out.println(getMksErrorMessage());
			assertTrue(true);
		}
	}

	public void testFileUnderDirNotInSandbox() throws TriclopsException {
		try {
			TriclopsSiMember member = getMksMember(FILE_UNDER_DIR_NOT_IN_SANDBOX_MKS);
			assertFalse(member.isStatusControlled());
			assertTrue(member.isStatusNotControlled());
		} catch (TriclopsException e) {
			System.out.println(getMksErrorMessage());
			fail();
		}
	}


	private TriclopsSiMember testFileUnderSandbox(String file) throws TriclopsException {
		sandbox.setIdeProjectPath(file);
		sandbox.validate();
		System.out.println("sandbox.getSandboxProject " + sandbox.getSandboxProject());
		TriclopsSiMembers members = new TriclopsSiMembers(CLIENT, sandbox);
		members.addMember(new TriclopsSiMember(file));
		members.getMembersStatus();
		debugMember(members.getMember(0));
		return members.getMember(0);
	}

	public void testFileInSandbox() throws TriclopsException {
		String file = FILE_IN_SANDBOX_MKS;
		assertTrue(testFileExistsInSandbox(file));
	}

	public void testFileUnderSandboxNotInSandbox() throws TriclopsException {
		String file = FILE_UNDER_NOT_IN_SANDBOX_MKS;
		testFileUnderSandbox(file);
		assertFalse(testFileExistsInSandbox(file));
	}

	/**
	 * @param file
	 * @return
	 * @throws TriclopsException
	 */
	private boolean testFileExistsInSandbox(String file) throws TriclopsException {
		TriclopsSiMember newMember = getMksMember(file);
		debugMember(newMember);
		return newMember.isStatusControlled();
	}

	private TriclopsSiMember getMksMember(String file) throws TriclopsException {
		testFileUnderSandbox(file);
		TriclopsSiMembers members;
		members = new TriclopsSiMembers(CLIENT, sandbox);
		TriclopsSiMember triclopsSiMember = new TriclopsSiMember(file);
		members.addMember(triclopsSiMember);
		members.getMembersStatus();
		TriclopsSiMember newMember = members.getMember(0);
		return newMember;
	}

	private void debugMember(TriclopsSiMember newMember) {
		if (DEBUG) {
			System.out.println("isStatusCanCheckIn : " + newMember.isStatusCanCheckIn());
			System.out.println("isStatusCanCheckOut : " + newMember.isStatusCanCheckOut());
			System.out.println("isStatusControlled : " + newMember.isStatusControlled());
			System.out.println("isStatusDifferent : " + newMember.isStatusDifferent());
			System.out.println("isStatusKnown : " + newMember.isStatusKnown());
			System.out.println("isStatusLocked : " + newMember.isStatusLocked());
			System.out.println("isStatusLockedByOther : " + newMember.isStatusLockedByOther());
			System.out.println("isStatusLockedByUser : " + newMember.isStatusLockedByUser());
			System.out.println("isStatusNewRevisionAvail : " + newMember.isStatusNewRevisionAvail());
			System.out.println("isStatusNotAuthorized : " + newMember.isStatusNotAuthorized());
			System.out.println("isStatusNotControlled : " + newMember.isStatusNotControlled());
			System.out.println("isStatusNoWorkingFile : " + newMember.isStatusNoWorkingFile());
			System.out.println("isStatusOutOfDate : " + newMember.isStatusOutOfDate());
			System.out.println("getArchive : " + newMember.getArchive());
			System.out.println("getArgFlags : " + newMember.getArgFlags());
			System.out.println("getLocker : " + newMember.getLocker());
			System.out.println("getPath : " + newMember.getPath());
			System.out.println("getRevision : " + newMember.getRevision());
			System.out.println("getStatus : " + newMember.getStatus());
			System.out.println("getStringArg : " + newMember.getStringArg());
		}
	}

	public void testFileNotInSandbox() throws TriclopsException {
		try {
			testFileUnderSandbox(FILE_NOT_IN_SANDBOX_MKS);
		} catch (TriclopsException e) {
			System.out.println(getMksErrorMessage());
			assertTrue(true);
		}
	}

	public void testDirInSandbox() throws TriclopsException {
		testFileUnderSandbox(DIR_IN_SANDBOX_MKS);
		assertTrue(true);
	}

	public void testDirInSandboxNotControlled() throws TriclopsException {
		testFileUnderSandbox(DIR_IN_SANDBOX_MKS_NOT_CONTROLLED);
		assertTrue(true);
	}

	public void testDirWithSlashInSandbox() throws TriclopsException {
		testFileUnderSandbox(DIR_IN_SANDBOX_MKS_WITH_SLASH);
		assertTrue(true);
	}


	public void testFileInSandboxLocallyDeleted() throws TriclopsException {
		TriclopsSiMember newMember = getMksMember(FILE_IN_SANDBOX_LOCALLY_DELETED_MKS);
		debugMember(newMember);
	}

	public void testFileInSandboxModified() throws TriclopsException {
		TriclopsSiMember newMember = getMksMember(FILE_IN_SANDBOX_MODIFIED_MKS);
		debugMember(newMember);
	}

	public void testSandbox() throws TriclopsException {
		String file = FILE_IN_SANDBOX_MKS;
		testFileUnderSandbox(file);
		String FILE_IN_SANDBOX_MKS_ProjectPath = sandbox.getIdeProjectPath();
		System.out.println("FILE_IN_SANDBOX_MKS => ");
		debugSandbox(sandbox);
		file = FILE_IN_SANDBOX_MODIFIED_MKS;
		testFileUnderSandbox(file);
		String FILE_IN_SANDBOX_MODIFIED_MKS_ProjectPath = sandbox.getIdeProjectPath();
		System.out.println("FILE_IN_SANDBOX_MODIFIED_MKS => ");
		debugSandbox(sandbox);
	}

	public void testSandboxOnProjectFile() throws TriclopsException {
		String file = FILE_IN_SANDBOX_MKS;
		sandbox.setIdeProjectPath(SANDBOX_ROOT + "/HJF-Core/project.pj");
//		testFileUnderSandbox(file);
//		file = sandbox.getPath();
//		sandbox.setIdeProjectPath(file);
		sandbox.validate();
		TriclopsSiMembers members;
		members = new TriclopsSiMembers(CLIENT, sandbox);
		TriclopsSiMember triclopsSiMember = new TriclopsSiMember(FILE_IN_SANDBOX_MKS);
		members.addMember(triclopsSiMember);
		members.getMembersStatus();
		debugMember(members.getMember(0));
		debugSandbox(sandbox);
	}

	private void debugSandbox(TriclopsSiSandbox sandbox) {
		if (DEBUG) {
			System.out.println("getCwd => " + sandbox.getCwd());
			System.out.println("getPath => " + sandbox.getPath());
			System.out.println("getSandboxProject => " + sandbox.getSandboxProject());
			System.out.println("getSandboxServer => " + sandbox.getSandboxServer());
			System.out.println("getSandboxType => " + sandbox.getSandboxType());
			System.out.println("getStatus => " + sandbox.getStatus());
			System.out.println("getTypeInfo => " + sandbox.getTypeInfo());
			System.out.println("isValid => " + sandbox.isValid());
			System.out.println("toString => " + sandbox);
		}
	}

	public void testProject() throws TriclopsException {
		sandbox.setIdeProjectPath(SANDBOX_ROOT + "/HJF-Core/project.pj");
		sandbox.validate();
		TriclopsSiProject proj = new TriclopsSiProject(CLIENT);
		System.out.println("sandbox getPath" + sandbox.getPath());
		System.out.println("sandbox getIdeProjectPath" + sandbox.getIdeProjectPath());
		System.out.println("sandbox getCwd" + sandbox.getCwd());

		proj.setPath(sandbox.getSandboxProject());
		String[] memberList = proj.getMemberList();
		for (String s : memberList) {
			System.out.println(s);
		}

	}
//	public void testFileInSandboxOldRevision() {
//		TriclopsSiMember newMember = getMksMember(FILE_IN_SANDBOX_OLD_REVISION_MKS);
//		debugMember(newMember);
//	}

	public String getMksErrorMessage() {
		return "Mks.errorMessage = " + CLIENT.getErrorMessage() +
			"\nMks.extendedErrorMessage = " + CLIENT.getExtendedErrorMessage();
	}
}
